@@ -79,9 +79,40 @@ CREATED_SOURCE_SYSTEM = dm.ContainerApply(
         )
     },
     constraints={"sourceUnique": dm.UniquenessConstraint(["source"])},
+    indexes={
+        "source": BTreeIndex(["source"], cursorable=True),
+    },
 )
 
-CONTAINERS = [RESOURCE_VIEW_MAPPING, INSTANCE_SOURCE_CONTAINER, CREATED_SOURCE_SYSTEM]
+SPACE_SOURCE = dm.ContainerApply(
+    space=SPACE.space,
+    external_id="SpaceSource",
+    used_for="node",
+    properties={
+        "instanceSpace": dm.ContainerProperty(
+            type=dm.Text(max_text_size=64),
+            nullable=False,
+        ),
+        "dataSetId": dm.ContainerProperty(
+            type=dm.data_types.Int64(),
+            nullable=False,
+        ),
+        "dataSetExternalId": dm.ContainerProperty(
+            type=dm.Text(max_text_size=256),
+            nullable=True,
+        ),
+    },
+    indexes={
+        "space": BTreeIndex(["instanceSpace"], cursorable=True),
+        "dataSetId": BTreeIndex(["dataSetId"], cursorable=True),
+        "dataSetExternalId": BTreeIndex(["dataSetExternalId"], cursorable=True),
+    },
+    constraints={
+        "dataSetIdUnique": dm.UniquenessConstraint(["dataSetId"]),
+    },
+)
+
+CONTAINERS = [RESOURCE_VIEW_MAPPING, INSTANCE_SOURCE_CONTAINER, CREATED_SOURCE_SYSTEM, SPACE_SOURCE]
 
 RESOURCE_VIEW_MAPPING_VIEW = dm.ViewApply(
     space=SPACE.space,
@@ -154,11 +185,37 @@ CREATED_SOURCE_SYSTEM_VIEW = dm.ViewApply(
     },
 )
 
+SPACE_SOURCE_VIEW = dm.ViewApply(
+    space=SPACE.space,
+    external_id="SpaceSource",
+    version="v1",
+    name="SpaceSource",
+    description="The mapping from CDF spaces to data sets.",
+    properties={
+        "instanceSpace": dm.MappedPropertyApply(
+            container=SPACE_SOURCE.as_id(),
+            container_property_identifier="instanceSpace",
+            description="The identifier of the created instances space.",
+        ),
+        "dataSetId": dm.MappedPropertyApply(
+            container=SPACE_SOURCE.as_id(),
+            container_property_identifier="dataSetId",
+            description="The dataSetId the space was created from.",
+        ),
+        "dataSetExternalId": dm.MappedPropertyApply(
+            container=SPACE_SOURCE.as_id(),
+            container_property_identifier="dataSetExternalId",
+            description="The externalId of the dataSet (if present) the space was created from.",
+        ),
+    },
+)
+
 INSTANCE_SOURCE_VIEW_ID = INSTANCE_SOURCE_VIEW.as_id()
 RESOURCE_VIEW_MAPPING_VIEW_ID = RESOURCE_VIEW_MAPPING_VIEW.as_id()
 CREATED_SOURCE_SYSTEM_VIEW_ID = CREATED_SOURCE_SYSTEM_VIEW.as_id()
+SPACE_SOURCE_VIEW_ID = SPACE_SOURCE_VIEW.as_id()
 
-VIEWS = [RESOURCE_VIEW_MAPPING_VIEW, INSTANCE_SOURCE_VIEW, CREATED_SOURCE_SYSTEM_VIEW]
+VIEWS = [RESOURCE_VIEW_MAPPING_VIEW, INSTANCE_SOURCE_VIEW, CREATED_SOURCE_SYSTEM_VIEW, SPACE_SOURCE_VIEW]
 
 COGNITE_MIGRATION_MODEL = dm.DataModelApply(
     space=SPACE.space,
@@ -166,7 +223,7 @@ COGNITE_MIGRATION_MODEL = dm.DataModelApply(
     version="v1",
     name="CDF Migration Model",
     description="Data model for migrating asset-centric resources to data modeling resources in CDF.",
-    views=[INSTANCE_SOURCE_VIEW_ID, RESOURCE_VIEW_MAPPING_VIEW_ID, CREATED_SOURCE_SYSTEM_VIEW_ID],
+    views=[INSTANCE_SOURCE_VIEW_ID, RESOURCE_VIEW_MAPPING_VIEW_ID, CREATED_SOURCE_SYSTEM_VIEW_ID, SPACE_SOURCE_VIEW_ID],
 )
 
 MODEL_ID = COGNITE_MIGRATION_MODEL.as_id()
