@@ -1,11 +1,14 @@
 from abc import ABC
 from pathlib import Path
-from typing import Literal
+from typing import Any, Literal
+
+from cognite_toolkit._cdf_tk.utils.useful_types import AssetCentricKind
 
 from ._base import DataSelector
 
 
-class AssetCentricSelector(DataSelector, ABC): ...
+class AssetCentricSelector(DataSelector, ABC):
+    kind: AssetCentricKind
 
 
 class DataSetSelector(AssetCentricSelector):
@@ -14,14 +17,16 @@ class DataSetSelector(AssetCentricSelector):
     type: Literal["dataSet"] = "dataSet"
 
     data_set_external_id: str
-    resource_type: str
 
     @property
     def group(self) -> str:
         return f"DataSet_{self.data_set_external_id}"
 
     def __str__(self) -> str:
-        return self.resource_type
+        return self.kind
+
+    def as_filter(self) -> dict[str, Any]:
+        return {"dataSetIds": [{"externalId": self.data_set_external_id}]}
 
 
 class AssetSubtreeSelector(AssetCentricSelector):
@@ -29,14 +34,16 @@ class AssetSubtreeSelector(AssetCentricSelector):
 
     type: Literal["assetSubtree"] = "assetSubtree"
     hierarchy: str
-    resource_type: str
 
     @property
     def group(self) -> str:
         return f"Hierarchy_{self.hierarchy}"
 
     def __str__(self) -> str:
-        return self.resource_type
+        return self.kind
+
+    def as_filter(self) -> dict[str, Any]:
+        return {"assetSubtreeIds": [{"externalId": self.hierarchy}]}
 
 
 class AssetCentricFileSelector(AssetCentricSelector):
