@@ -165,10 +165,16 @@ from cognite_toolkit._cdf_tk.client.data_classes.location_filters import (
     LocationFilterWriteList,
 )
 from cognite_toolkit._cdf_tk.client.data_classes.migration import (
-    ViewSource,
-    ViewSourceApply,
+    ResourceViewMapping,
+    ResourceViewMappingApply,
 )
 from cognite_toolkit._cdf_tk.client.data_classes.raw import RawDatabase, RawDatabaseList
+from cognite_toolkit._cdf_tk.client.data_classes.search_config import (
+    SearchConfig,
+    SearchConfigList,
+    SearchConfigWrite,
+    SearchConfigWriteList,
+)
 
 from .data_classes import APIResource, Method
 
@@ -800,10 +806,10 @@ API_RESOURCES = [
         },
     ),
     APIResource(
-        api_name="migration.view_source",
-        resource_cls=ViewSource,
-        list_cls=NodeList[ViewSource],
-        _write_cls=ViewSourceApply,
+        api_name="migration.resource_view_mapping",
+        resource_cls=ResourceViewMapping,
+        list_cls=NodeList[ResourceViewMapping],
+        _write_cls=ResourceViewMappingApply,
         _write_list_cls=NodeApplyList,
         methods={
             "create": [Method(api_class_method="upsert", mock_class_method="create_nodes")],
@@ -814,4 +820,17 @@ API_RESOURCES = [
             ],
         },
     ),
+    APIResource(
+        api_name="search.configurations",
+        resource_cls=SearchConfig,
+        list_cls=SearchConfigList,
+        _write_cls=SearchConfigWrite,
+        _write_list_cls=SearchConfigWriteList,
+        methods={
+            "create": [Method(api_class_method="upsert", mock_class_method="create_multiple")],
+            "retrieve": [
+                Method(api_class_method="list", mock_class_method="return_values"),
+            ],
+        },
+    ),
 ]
