@@ -57,7 +57,7 @@ class TestAssetIO:
     ) -> None:
         config = toolkit_config
         asset_by_external_id = {asset.external_id: asset for asset in some_asset_data if asset.external_id is not None}
-        selector = AssetSubtreeSelector(hierarchy="test_hierarchy", resource_type="asset")
+        selector = AssetSubtreeSelector(hierarchy="test_hierarchy", kind="Assets")
 
         def create_callback(request: httpx.Request) -> httpx.Response:
             payload = json.loads(request.content)
@@ -85,8 +85,9 @@ class TestAssetIO:
 
             source = io.stream_data(selector)
             json_chunks: list[list[dict[str, JsonVal]]] = []
-            for chunk in source:
-                json_chunk = io.data_to_json_chunk(chunk)
+            for page in source:
+                # New interface: stream_data returns Page objects
+                json_chunk = io.data_to_json_chunk(page.items)
                 assert isinstance(json_chunk, list)
                 assert len(json_chunk) == 10
                 for item in json_chunk:
@@ -96,9 +97,13 @@ class TestAssetIO:
                 json_chunks.append(json_chunk)
 
             with HTTPClient(config) as upload_client:
-                data_chunks = (io.json_chunk_to_data(chunk) for chunk in json_chunks)
-                for data_chunk in data_chunks:
-                    io.upload_items(data_chunk, upload_client, selector)
+                from cognite_toolkit._cdf_tk.storageio._base import UploadItem
+
+                # New interface: convert individual items and create UploadItems
+                for chunk in json_chunks:
+                    write_items = [io.json_to_resource(item) for item in chunk]
+                    upload_items = [UploadItem(source_id=io.as_id(item), item=item) for item in write_items]
+                    io.upload_items(upload_items, upload_client, selector)
 
             assert respx_mock.calls.call_count == 10  # 100 rows in chunks of 10
             uploaded_assets = []
@@ -130,7 +135,7 @@ class TestAssetIO:
 
         respx_mock.post(config.create_api_url("/assets")).mock(side_effect=asset_create_callback)
 
-        selector = AssetSubtreeSelector(hierarchy="test_hierarchy", resource_type="asset")
+        selector = AssetSubtreeSelector(hierarchy="test_hierarchy", kind="Assets")
         with monkeypatch_toolkit_client() as client:
             client.config = config
             client.verify.authorization.return_value = []
@@ -215,7 +220,7 @@ class TestFileMetadataIO:
             )
 
         respx_mock.post(config.create_api_url("/files")).mock(side_effect=create_callback)
-        selector = DataSetSelector(data_set_external_id="DataSetSelector", resource_type="file")
+        selector = DataSetSelector(data_set_external_id="DataSetSelector", kind="FileMetadata")
 
         with monkeypatch_toolkit_client() as client:
             client.config = config
@@ -232,8 +237,9 @@ class TestFileMetadataIO:
 
             source = io.stream_data(selector)
             json_chunks: list[list[dict[str, JsonVal]]] = []
-            for chunk in source:
-                json_chunk = io.data_to_json_chunk(chunk)
+            for page in source:
+                # New interface: stream_data returns Page objects
+                json_chunk = io.data_to_json_chunk(page.items)
                 assert isinstance(json_chunk, list)
                 assert len(json_chunk) == 10
                 for item in json_chunk:
@@ -243,9 +249,13 @@ class TestFileMetadataIO:
                 json_chunks.append(json_chunk)
 
             with HTTPClient(config) as upload_client:
-                data_chunks = (io.json_chunk_to_data(chunk) for chunk in json_chunks)
-                for data_chunk in data_chunks:
-                    io.upload_items(data_chunk, upload_client, selector)
+                from cognite_toolkit._cdf_tk.storageio._base import UploadItem
+
+                # New interface: convert individual items and create UploadItems
+                for chunk in json_chunks:
+                    write_items = [io.json_to_resource(item) for item in chunk]
+                    upload_items = [UploadItem(source_id=io.as_id(item), item=item) for item in write_items]
+                    io.upload_items(upload_items, upload_client, selector)
 
             # /files only support creating one at a time.
             assert respx_mock.calls.call_count == len(some_filemetadata_data)
@@ -286,7 +296,7 @@ class TestTimeSeriesIO:
     ) -> None:
         config = toolkit_config
         ts_by_external_id = {ts.external_id: ts for ts in some_timeseries_data if ts.external_id is not None}
-        selector = DataSetSelector(data_set_external_id="DataSetSelector", resource_type="timeseries")
+        selector = DataSetSelector(data_set_external_id="DataSetSelector", kind="TimeSeries")
 
         def create_callback(request: httpx.Request) -> httpx.Response:
             payload = json.loads(request.content)
@@ -316,8 +326,9 @@ class TestTimeSeriesIO:
 
             source = io.stream_data(selector)
             json_chunks: list[list[dict[str, JsonVal]]] = []
-            for chunk in source:
-                json_chunk = io.data_to_json_chunk(chunk)
+            for page in source:
+                # New interface: stream_data returns Page objects
+                json_chunk = io.data_to_json_chunk(page.items)
                 assert isinstance(json_chunk, list)
                 assert len(json_chunk) == 10
                 for item in json_chunk:
@@ -327,9 +338,13 @@ class TestTimeSeriesIO:
                 json_chunks.append(json_chunk)
 
             with HTTPClient(config) as upload_client:
-                data_chunks = (io.json_chunk_to_data(chunk) for chunk in json_chunks)
-                for data_chunk in data_chunks:
-                    io.upload_items(data_chunk, upload_client, selector)
+                from cognite_toolkit._cdf_tk.storageio._base import UploadItem
+
+                # New interface: convert individual items and create UploadItems
+                for chunk in json_chunks:
+                    write_items = [io.json_to_resource(item) for item in chunk]
+                    upload_items = [UploadItem(source_id=io.as_id(item), item=item) for item in write_items]
+                    io.upload_items(upload_items, upload_client, selector)
 
             assert respx_mock.calls.call_count == 5  # 50 rows in chunks of 10
             uploaded_ts = []
@@ -368,7 +383,7 @@ class TestEventIO:
     ) -> None:
         config = toolkit_config
         event_by_external_id = {event.external_id: event for event in some_event_data if event.external_id is not None}
-        selector = DataSetSelector(data_set_external_id="DataSetSelector", resource_type="event")
+        selector = DataSetSelector(data_set_external_id="DataSetSelector", kind="Events")
 
         def create_callback(request: httpx.Request) -> httpx.Response:
             payload = json.loads(request.content)
@@ -398,8 +413,8 @@ class TestEventIO:
 
             source = io.stream_data(selector)
             json_chunks: list[list[dict[str, JsonVal]]] = []
-            for chunk in source:
-                json_chunk = io.data_to_json_chunk(chunk)
+            for page in source:
+                json_chunk = io.data_to_json_chunk(page.items)
                 assert isinstance(json_chunk, list)
                 assert len(json_chunk) == 10
                 for item in json_chunk:
@@ -409,9 +424,13 @@ class TestEventIO:
                 json_chunks.append(json_chunk)
 
             with HTTPClient(config) as upload_client:
-                data_chunks = (io.json_chunk_to_data(chunk) for chunk in json_chunks)
-                for data_chunk in data_chunks:
-                    io.upload_items(data_chunk, upload_client, selector)
+                from cognite_toolkit._cdf_tk.storageio._base import UploadItem
+
+                # New interface: convert individual items and create UploadItems
+                for chunk in json_chunks:
+                    write_items = [io.json_to_resource(item) for item in chunk]
+                    upload_items = [UploadItem(source_id=io.as_id(item), item=item) for item in write_items]
+                    io.upload_items(upload_items, upload_client, selector)
 
             assert respx_mock.calls.call_count == 5  # 50 rows in chunks of 10
             uploaded_events = []
